/*
*     Copyright (C) 2002, Olexander O. Sudakov <csaa@mail.univ.kiev.ia>
*     Copyright (C) 2002, Eugeniy S. Meshcheryakov <eugen@univ.kiev.ua>
*
*     This program is free software; you can redistribute it and/or modify
*     it under the terms of the GNU General Public License as published by
*     the Free Software Foundation; either version 2 of the License, or
*     (at your option) any later version.
*
*     This program is distributed in the hope that it will be useful,
*     but WITHOUT ANY WARRANTY; without even the implied warranty of
*     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*     GNU General Public License for more details.
*
*     You should have received a copy of the GNU General Public License
*     along with this program; if not, write to the Free Software
*     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**/

#ifndef __CHPOX_H__
#define __CHPOX_H__


/*Boroowed from CRACK */
#include "chpoxconf.h"

#include <asm/ptrace.h>
#include <linux/ipc.h>
#include <asm/param.h>

/* Type of file descriptor encountered: */
#define CKPT_DUP                        0
#define CKPT_FILE                       1
#define CKPT_PIPE                       2
#define CKPT_SOCK                       3
#define CKPT_OTHER                      4

#define IN_CHPOX_NOTIFIER 0x100


struct open_files_hdr {
	        int number_open_files;
};

struct ckpt_fdcache_struct {
	  int fd;
	  struct inode *inode;
	  struct file *file;
};

struct open_files {
  int entry_size; /* How many bytes are we dumping after this struct */
  int type; /* CKPT_PIPE, CKPT_FILE,CKPT_SOCKET */
  int fd; /* Original fd */
  union
  {
	      // this fd is a dup of another fd
     struct {
           int dupfd;
     } dup;
     struct {
           unsigned long inode; /* just a unique identifier of inode */
//           int lock; /* Pipe lock */
//           unsigned blocked:1;
           unsigned rdwr:1; /* 0 - read, 1 - write */
     } pipes;
     struct {
           unsigned long int file_pos;
           unsigned long file; /* unique identifier of struct file */
           int flags;
           int mode;
           char *filename; /* see (1) */
      } file;
      struct {
           int domain;
           int type;
           int protocol;
           int state;
           int reuse;
           unsigned int saddr, daddr;
           unsigned sport, dport;
           unsigned short backlog;
      } sock;
  } u;
};
#if 0
/* used while dumping files */
struct ckpt_pipe_entry {
	struct list_head list;
	unsigned long nrefs_orig;
	struct inode *inode; /* inode reference */
	unsigned long nrefs; /* sum of number of readers and writers */
};

/* used while restoring files */
struct ckpt_pipe_restore_entry {
	struct list_head list;
	unsigned long nrefs;
	unsigned long ino;
	struct file *r_file;
	struct file *w_file;
};
#endif 

 /* Check whether the specified task has the fd open. Since the task
  * may not have a files_struct, we must test for p->files != NULL.
  *
  * copied from linux/file.h
 */
static inline struct file * fcheck_task(struct task_struct *p, unsigned int fd) {
         struct file * file = NULL;

         if (p->files && fd < p->files->max_fds)
              file = p->files->fd[fd];
    return file;
 }

static inline struct inode *
ckpt_icheck_task(struct task_struct *p, unsigned int fd) {

  struct file *fdes = fcheck_task(p, fd);
  struct dentry *dent = fdes ? fdes->f_dentry : NULL;
  struct inode *inode = dent ? dent->d_inode : NULL;
  return inode;
}

/* End borrowed from crack */

struct ckpt_data {
  struct list_head list;      /* list of such items */
  // struct list_head children;  /* list of child processes */  
  int pid;                    /* pid to ckpt */
  int interval;               /* ckpt parametr */
  int flags;                  /* flags indicates current state */
  int signal;                 /* signal to use for checkpoint */
  struct file* file;          /* file to dump to */
  char* file_name;            /* name of file to dump */
  int fname_size;	      /* length of file name*/
  void* old_handler;          /* old signal handler that should be restored */
  sigset_t mask;              /* mask for signal blocking */
  int nt;                     /* number of times notifier called */
  struct timeval tv;          /* the last ckpt time */
  //DECLARE_RWSEM(lock);         /* locking */
  int count;                  /* refference count */
  struct ckpt_data* parent;   /* parent process data*/
 
  
};

#if 0
struct pipe_cache_item {
    struct list_head list;
    unsigned long old_inode;
    unsigned long new_inode;
    pid_t first_pid;
};
#endif 

#define IS_FD_NAMED(fdes)       ((fdes)->f_dentry->d_name.name)
#define OPEN_SAFE(fd, filename) (fd>=3 || ( strncmp("/dev/tty",filename,8) && strncmp("/dev/pts",filename,8) && strncmp("/dev/vc/",filename,8) ))
enum {CKPT_CREATED,CKPT_SIGNALED,CKPT_DONE,CKPT_FAILED};


#define DUMP_PROC_DIR "chpox"
#define DUMP_PROC_REGISTER "register"
#define DUMP_PROC_INFO "info"
#define DUMP_PROC_CONF "prefix"

#endif
